@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import shelve
+import shutil
 from cmd import Cmd
 from collections import Counter
 from enum import Enum
@@ -156,18 +157,19 @@ def _tile_display(tiles: Iterable[str]) -> str:
     displayed. A 2-space indent is added to all lines.
 
     """
-    width = click.get_terminal_size()[0] - 4  # 2 space indent, 2 spaces margin
+    width = shutil.get_terminal_size()[0] - 4  # 2 space indent, 2 spaces margin
     if width < 3:  # obviously too narrow for even a single tile, ignore
         width = 76
     line: list[str] = []
     lines, used = [line], 0
     for tile in tiles:
-        if used - 1 >= width:  # do not count a space at the end
+        tl = len(tile) + 2  # includes comma and space
+        if used + tl - 1 >= width:  # do not count a space at the end
             # start a new line
             used, line = 0, []
             lines.append(line)
         line.append(Colours.c(tile))
-        used += len(tile) + 2
+        used += tl
     return "  " + ",\n  ".join([", ".join(ln) for ln in lines])
 
 
@@ -274,7 +276,7 @@ class SolverConsole(Cmd):
 
     def message(self, *msg: object, wrap=False, perhaps_paged=False) -> None:
         text = " ".join(map(str, msg))
-        twidth, theight = click.get_terminal_size()
+        twidth, theight = shutil.get_terminal_size()
         if wrap:
             text = click.wrap_text(text, width=twidth, preserve_paragraphs=True)
         if perhaps_paged and len(text.splitlines()) >= theight:
@@ -625,6 +627,33 @@ class SolverConsole(Cmd):
     emptyline = do_solve
     complete_solve = _fixed_completer("tiles", "value", "initial")
 
+    def do_check(self, arg: str) -> None:
+        """check
+        Check the table for validity and set placement
+
+        Either shows you a valid arrangement of the tiles on the board, or
+        if no such arrangement can be made, reports this. It also reports on
+        the number of jokers that can be taken without invalidating the
+        table.
+
+        """
+        arr = self._ruleset.arrange_table(self.game)
+        if not arr:
+            self.message(
+                click.style(
+                    "The tiles on the table can't be formed into valid sets",
+                    fg="yellow",
+                )
+            )
+            return
+
+        self.message("Possible table arrangement:")
+        for s in arr.sets:
+            self.message(" ", ", ".join([Colours.c(self._r_tile_map[t]) for t in s]))
+        if arr.free_jokers:
+            jokers = ", ".join([Colours.c(JOKER)] * arr.free_jokers)
+            self.message(click.style(f"Free jokers: {jokers}", fg="bright_green"))
+
     def do_stop(self, arg: str) -> bool:
         """stop | end | quit
         Exit from the console
@@ -664,7 +693,7 @@ class SolverConsole(Cmd):
 
     def help_tiles(self) -> None:
         cols = chain(islice(Colours, self._ruleset.colours), (Colours.joker,))
-        width = click.get_terminal_size()[0]
+        width = shutil.get_terminal_size()[0]
         # there is no easy option to rewrap text _with ANSI escapes_, so
         # rewrap text in sections.
         dedent_and_wrap = lambda t: click.wrap_text(  # noqa: E731
