@@ -1,7 +1,7 @@
 # SPDX-License-Identifier: MIT
 from __future__ import annotations
 from collections import Counter
-from typing import Iterable, Sequence
+from typing import Optional, Sequence
 
 import numpy as np
 
@@ -26,36 +26,48 @@ class GameState:
     rack_array: np.array[np.int8]  # array with per-tile counts on the rack
     table_array: np.array[np.int8]  # array with per-tile counts on the table
 
-    def __init__(self, tile_count: int) -> None:
+    def __init__(
+        self,
+        tile_count: int,
+        table: Optional[Sequence[int]] = None,
+        rack: Optional[Sequence[int]] = None,
+    ) -> None:
         self._tile_count = tile_count
         self.reset()
+        if table:
+            self.add_table(table)
+        if rack:
+            self.add_rack(rack)
 
     def reset(self) -> None:
         self.table, self.rack, self.initial = Counter(), Counter(), True
         self.table_array = np.zeros(self._tile_count, dtype=np.int8)
         self.rack_array = np.zeros(self._tile_count, dtype=np.int8)
 
+    def __repr__(self) -> str:
+        return (
+            f"{type(self).__name__}({self._tile_count}, "
+            f"table={tuple(self.sorted_table)} rack={tuple(self.sorted_rack)})"
+        )
+
     def with_move(self, tiles: Sequence[int]) -> GameState:
         """New state with tiles moved
 
-        The tiles are assumed to be on the rack and are moved to the table.
+        The tiles are verified to be on the rack and are moved to the table.
         This doesn't mutate this state but rather creates a new state with
         the new tile locations
 
         """
-        clone = self.__new__(type(self))
-        clone._tile_count = self._tile_count
+        moved = Counter(tiles)
+        if moved - self.rack:
+            raise ValueError("Move includes tiles not on the rack")
+        table = list((self.table + moved).elements())
+        rack = list((self.rack - moved).elements())
+        return type(self)(self._tile_count, table, rack)
 
-        tcounts = Counter(tiles)
-        clone.table = self.table + tcounts
-        clone.rack = self.rack - tcounts
-
-        array = np.zeros_like(self.table_array)
-        np.add.at(array, np.array(tiles) - 1, 1)
-        clone.table_array = np.add(self.table_array, array)
-        clone.rack_array = np.subtract(self.rack_array, array)
-
-        return clone
+    def table_only(self) -> GameState:
+        """New state with just the table tiles"""
+        return type(self)(self._tile_count, self.sorted_table)
 
     @property
     def sorted_rack(self) -> list[int]:
@@ -67,7 +79,7 @@ class GameState:
         """The table tile numbers as a sorted list"""
         return sorted(self.table.elements())
 
-    def add_rack(self, additions: Iterable[int]) -> None:
+    def add_rack(self, additions: Sequence[int]) -> None:
         if not additions:
             return
         self.rack += Counter(additions)
@@ -81,7 +93,7 @@ class GameState:
         np.subtract.at(rack, np.array(removals) - 1, 1)
         rack[rack < 0] = 0  # in case we removed tiles not on the rack
 
-    def add_table(self, additions: Iterable[int]) -> None:
+    def add_table(self, additions: Sequence[int]) -> None:
         if not additions:
             return
         self.table += Counter(additions)
